 # -*- coding: utf-8 -*-
r'''Extract an image for each image in a SPOT archive.

Usage: SPOT-zip-to-preview.py [in_process_list.csv]  [out_folder] [max_dimension]

    in_process_list.csv : "A:\path\to, DS_archive_name_01, .zip" per line
    out_folder          : self explanatory
    max_dimension       : Output pixel width/height maximum constraint
                          Use zero (0) to keep original dimensions.

Typical output is:

    - one 4 channel RGB-NearInfrared image (image_name_16bit.tif)
    - one single channel pan-chromatic image (image_name_pan.tif)

    - name_A_16bit.tif - "A" is primary camera/sensor, "B" is 20° stereo pair

Assumptions are made about which files within the archive contain data, deduced 
from the files I have on hand. (There's no guarantee we get all the images.)

Known data we don't extract or do anything with:
    - License and other metadata
    - LINEAGE
    - MASKS

The band descriptions are hard coded from what I read. It would be better to 
look them up in a table or something based on metadata.
https://www.satimagingcorp.com/satellite-sensors/spot-6/

We read input from a csv file. The odd format is a hold-over from earlier scripts.
It would be better to just follow standard "verb in-file out-file" convention and 
leave grouping up batch inputs to something else.

Matt.Wilkie@yukon.ca, 2021-Oct-14
(c) 2021 Yukon Government, X/MIT Open Source license.
'''
import os
import csv
import sys
import shutil
from pathlib import Path
from zipfile import ZipFile, BadZipFile
from osgeo import gdal

gdal.UseExceptions()

if len(sys.argv) < 4:
    print(f"Usage: {sys.argv[0]} [in_process_list.csv]  [out_folder] [max_dimension]")
    sys.exit(1)

src = sys.argv[1]
out = sys.argv[2]
max_dimension = int(sys.argv[3]) # max num pixels in any dimension

## OPTIONS
# max memory consumption
gdal.SetConfigOption('GDAL_CACHEMAX','30%')
# gdal Creation Options per driver format (https://gdal.org/drivers/raster/cog.html)
options = ["COMPRESS=ZSTD", "PREDICTOR=YES", "LEVEL=17", "BIGTIFF=YES",
    "NUM_THREADS=ALL_CPUS",]

if not os.path.exists(out):
    os.makedirs(out)

def progress_cb(complete, message, cb_data):
    '''Emit progress report in numbers for 10% intervals and dots for 3%'''
    if int(complete*100) % 10 == 0:
        print(f'{complete*100:.0f}', end='', flush=True)
    elif int(complete*100) % 3 == 0:
        print(f'{cb_data}', end='', flush=True)


with open(src) as csvfile:
    reader = csv.reader(csvfile)
    for r in reader:
        # \\server\share\\ ...\Raw\\DS_SPOT6_201809291942542_LM1_FR1_FR1_FR1_W131N61_04875_A.zip
        srcname = f"{r[0]}\{r[1]}{r[2]}"
        if os.path.exists(srcname):
            try:
                with ZipFile(srcname) as zip:
                    # identify the Dataset index file
                    dslist = [x for x in zip.namelist() if "/DIM" in x]
                    for ds in dslist:

                        # VOL_SPOT6_001_A or VOL_SPOT6_001_B?
                        v = ""
                        vol = [x for x in ds.split("/") if "VOL_" in x][0]
                        if "_A" in vol:
                            v = "_A"
                        elif "_B" in vol:
                            v = "_B"

                        # multispectral (RGB-Nir)
                        if "_MS_" in ds:
                            dstname = os.path.join(out,f"{r[1]}{v}_16bit.tif")
                            options.append("PHOTOMETRIC=RGB")
                        # panchromatic band
                        if "_P_" in ds:
                            dstname = os.path.join(out,f"{r[1]}{v}_pan.tif")

                        if not os.path.exists(dstname):
                            vsipath = f"/vsizip/{srcname}/{ds}"
                            print(vsipath)

                            data_in = gdal.Open(vsipath)
                            # print(data_in.GetMetadata())

                            if data_in.RasterXSize > data_in.RasterYSize:
                                if max_dimension == 0:
                                    max_dimension = data_in.RasterXSize
                                w,h = max_dimension,0
                            else:
                                if max_dimension == 0:
                                    max_dimension = data_in.RasterYSize
                                w,h = 0,max_dimension

                            tmp_vrt = gdal.Translate("", data_in, format = 'VRT')
                            if "_MS_" in ds:
                                tmp_vrt.GetRasterBand(1).SetColorInterpretation(gdal.GCI_RedBand)
                                tmp_vrt.GetRasterBand(1).SetDescription("Red (0.625 - 0.695 microns)")
                                tmp_vrt.GetRasterBand(2).SetColorInterpretation(gdal.GCI_GreenBand)
                                tmp_vrt.GetRasterBand(2).SetDescription("Green (0.530 - 0.590 microns)")
                                tmp_vrt.GetRasterBand(3).SetColorInterpretation(gdal.GCI_BlueBand)
                                tmp_vrt.GetRasterBand(3).SetDescription("Blue (0.455 - 0.525 microns)")
                                #data_in.GetRasterBand(4).SetColorInterpretation("Gray")
                                tmp_vrt.GetRasterBand(4).SetDescription("Near-Infrared (0.760 - 0.890 microns)")
                            if "_P_" in ds:
                                #data_in.GetRasterBand(1).SetColorInterpretation("Gray")
                                tmp_vrt.GetRasterBand(1).SetDescription("Panchromatic")

                            gdal.Translate(dstname, tmp_vrt, width=w, height=h,
                                format="COG",
                                noData=0,
                                creationOptions=options,
                                callback=progress_cb,
                                callback_data='.')

                            # close images
                            data_in = None
                            tmp_vrt = None
            except (BadZipFile,RuntimeError) as e:
                print(f"*** bad zip: {srcname}\n*** {e}")
